/*
	SDL LDG loader

	Copyright (C) 2004	Patrice Mandin

	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
	   claim that you wrote the original software. If you use this software
	   in a product, an acknowledgment in the product documentation would be
	   appreciated but is not required.
	2. Altered source versions must be plainly marked as such, and must not be
	   misrepresented as being the original software.
	3. This notice may not be removed or altered from any source distribution.
*/

/*--- Includes ---*/

#include <stdarg.h>
#include <stdlib.h>

#include <gem.h>
#include <ldg.h>

#define _SDL_error_h	/* don't include it */
#define DISABLE_X11		/* for SDL_syswm.h */

#include <SDL_audio.h>
#include <SDL_cdrom.h>
#include <SDL_events.h>
#include <SDL_joystick.h>
#include <SDL_keyboard.h>
#include <SDL_keysym.h>
#include <SDL_mouse.h>
#include <SDL_mutex.h>
#include <SDL_rwops.h>
#include <SDL_syswm.h>
#include <SDL_thread.h>
#include <SDL_timer.h>
#include <SDL_types.h>
#include <SDL_version.h>
#include <SDL_video.h>

/*--- Variables ---*/

static LDG *ldg_sdl=NULL;

/*--- Functions prototypes ---*/

static int sdl_ldg_loader(void);

/*--- SDL functions pointers from sdl.ldg ---*/

#include "pointers.h"

/*--- LDG loader ---*/

static int sdl_ldg_loader(void)
{
	/* Load the library */
	ldg_sdl = ldg_open("sdl.ldg", ldg_global);
	if (ldg_sdl == NULL) {
		return -1;
	}

	#include "loaders.h"

	return 0;
}

/*--- SDL functions redirectors ---*/

/* sdl.h */

int SDL_Init(Uint32 flags)
{
	if (ldg_sdl == NULL) {
		if (sdl_ldg_loader()<0) {
			return -1;
		};
	}
	return (*SDL_Init_p)(flags);
}

int SDL_InitSubSystem(Uint32 flags)
{
	return (*SDL_InitSubSystem_p)(flags);
}

void SDL_QuitSubSystem(Uint32 flags)
{
	(*SDL_QuitSubSystem_p)(flags);
}

Uint32 SDL_WasInit(Uint32 flags){
	return (*SDL_WasInit_p)(flags);
}

void SDL_Quit(void)
{
	(*SDL_Quit_p)();

	ldg_close(ldg_sdl, ldg_global);
	ldg_sdl = NULL;
}

#include "calls.h"
